angular.module('ionic-audio', ['ionic'])
  .filter('time', function ()
  {
    return function (input)
      {
        input = input || 0;

        var t = parseInt(input, 10);

        var addLeadingZero = function (n)
          {
            return (n < 10) ? '0' + n : n;
          };
        return addLeadingZero(Math.floor(t / 60)) + ':' + addLeadingZero(t % 60);
      };
  })
  .filter('duration', [
    '$filter', function ($filter)
      {
        return function (input)
          {
            return (input > 0) ? $filter('time')(input) : '';
          }
      }
  ])
  .factory('MediaManager', [
    '$interval', '$timeout', '$window', function ($interval, $timeout, $window)
      {
        var tracks = [], currentMedia, playerTimer;
        var currentTrack = {};
        var myStatus, mediaInterval;
        if (!$window.cordova && !$window.Media)
          {
            console.log("ionic-audio: missing Cordova Media plugin. Have you installed the plugin? \nRun 'ionic plugin add org.apache.cordova.media'");
            return null;
          }

        var startTimer = function ()
          {
            if (angular.isDefined(playerTimer)) return;

            if (!currentTrack) return;

            playerTimer = $interval(function ()
            {
              if (currentTrack.duration < 0)
                {
                  currentTrack.duration = currentMedia.getDuration();
                }

              currentMedia.getCurrentPosition(
                // success callback
                function (position)
                {
                  if (position > -1)
                    {
                      currentTrack.progress = position;
                    }
                },
                // error callback
                function (e)
                {
                  console.log("Error getting pos=" + e);
                });

              if (angular.isFunction(currentTrack.onProgress))
                currentTrack.onProgress(currentTrack.progress, currentTrack.duration);

            }, 1000);

          };

        var stopTimer = function ()
          {
            if (angular.isDefined(playerTimer))
              {
                $interval.cancel(playerTimer);
                playerTimer = undefined;
              }
          };

        var releaseMedia = function ()
          {
            if (angular.isDefined(currentMedia))
              {
                currentMedia.release();
                //currentMedia = undefined;
                //currentTrack = {};
              }
          };

        var onSuccess = function ()
          {
            stopTimer();
            releaseMedia();
            //console.info('this in onSuccess)', this);
            if (angular.isFunction(this.onSuccess))
              {
                this.onSuccess();
                kill();
              }
          };

        var onError = function ()
          {
            if (angular.isFunction(this.onError))
              this.onError();
          };

        var onStatusChange = function (status)
          {
            this.status = status;
            myStatus = this.status;
            currentTrack.status = status;
            console.log('current track status add on', currentTrack);
            if (angular.isFunction(this.onStatusChange))
              this.onStatusChange(status);
          };

        var createMedia = function (track)
          {
            if (!track)
              {
                console.log('ionic-audio: missing track url');
                return undefined;
              }

            return new Media(track.mp3,
              angular.bind(track, onSuccess),
              angular.bind(track, onError),
              angular.bind(track, onStatusChange));

          };

        var destroy = function ()
          {
            //stopTimer();
            releaseMedia();
          };

        var stop = function ()
          {
            console.log('ionic-audio: stopping track ' + currentTrack.mp3);
            currentMedia.stop();    // will call onSuccess...

            //currentTrack = {};
          };

        var pause = function ()
          {
            console.log('pause 2=========');

            console.log('ionic-audio: pausing track ' + currentTrack.mp3);

            currentMedia.pause();
            //stopTimer();
          };

        var resume = function ()
          {
            console.log('ionic-audio: resuming track ' + currentTrack);

            currentMedia.play();
            //startTimer();
          };

        var seekTo = function (pos)
          {
            if (!currentMedia) return;

            currentMedia.seekTo(pos * 1000);
          };

        var counter = 0;

        var kill = function ()
          {
            currentMedia.stop();
            currentMedia.release();
            currentTrack = {};
            myStatus = null;
            counter = 0;
          };
        var checkMedia = function ()
          {
            if (currentMedia) currentMedia.release();
            if (angular.isDefined(mediaInterval)) $interval.cancel(mediaInterval);
          };
        var pos = function (track, id)
          {

            console.log('track', track, 'counter', counter);

            if (track.length === counter)
              {
                console.warn('############## media Interval completed ############## ');
                $interval.cancel(mediaInterval);
                return;
                //kill();
              }


            currentTrack.mp3 = track[counter].mp3;

            if (!currentTrack.mp3)
              return;
            console.log('ionic-audio: playing track ' + currentTrack.mp3);
            console.log('my status inner', 'counter', counter, 'eval',
              (myStatus == '4'));
            console.warn('track length', track.length, 'counter', counter);


            if (myStatus)
              {
                var string = myStatus + '';
                console.log("PEev==" + string + " == " + ('4'.indexOf(string) != -1) + " - " + (myStatus == 4));
              }


            if (!myStatus)
              {
                currentMedia = createMedia(currentTrack);
                currentMedia.play();
                //startTimer();
                console.log('playlist item:' + counter + '===========', currentMedia);

                counter++;
              }
            else if (myStatus == 4)
              {

                currentMedia = createMedia(currentTrack);

                currentMedia.play();
                //startTimer();
                console.log('playlist item:' + counter + '===========', currentMedia);

                counter++;
              }
            console.log('counter++', counter);

          };
        var currentId;
        var stop = function ()
          {
            currentMedia.stop();
          };
        var play = function (track, id)
          {
            if (currentId)
              {

                if (currentId != id)
                  {
                    kill();
                  }
              }
            currentId = id;

            //$interval(pos, 500, 0, true, track);

            if (angular.isDefined(mediaInterval))
              {
                console.info('mediaInterval is defined ------ cleaning');
                currentMedia.stop();
                currentTrack = {};
                myStatus = null;
                counter = 0;
              }

            mediaInterval = $interval(function (track, id)
            {

              //console.info('============mediaInterval start============');
              pos(track);
            }, 500, 0, true, track);


            //currentTrack = track[0].mp3;
            //
            //console.log('ionic-audio: playing track ' + currentTrack);
            //
            //currentMedia = createMedia(currentTrack);
            //currentMedia.play();
            //
            //startTimer();

          };

        /*
         Creates a new Media from a track object

         var track = {
         url: 'https://s3.amazonaws.com/ionic-audio/Message+in+a+bottle.mp3',
         artist: 'The Police',
         title: 'Message in a bottle',
         art: 'img/The_Police_Greatest_Hits.jpg'
         }
         */
        return {
          add: function (track, playbackSuccess, playbackError, statusChange, progressChange)
            {
              if (!track)
                {
                  console.log('ionic-audio: missing track url');
                  return;
                }
              angular.extend(track, {
                onSuccess: playbackSuccess,
                onError: playbackError,
                onStatusChange: statusChange,
                onProgress: progressChange,
                status: 0,
                duration: -1,
                progress: 0
              });

              track.id = tracks.push(track) - 1;
              return track.id;
            },

          play: function (trackID)
            {
              console.info('currentmedia', (currentMedia) ? currentMedia : 'undef', 'currentTrack',
                (currentTrack) ? currentTrack : 'undef', 'status', (myStatus) ? myStatus : 'undef', 'trackID',
                (trackID) ? trackID : 'undef');
              // avoid two tracks playing simultaneously
              if (currentTrack)
                {
                  console.log('current track id', currentTrack.id, 'trackID', trackID, 'status', currentTrack.status);

                  if (currentMedia && currentTrack.status == 2)
                    {
                      console.log('pause 3=========');

                      pause();
                    }
                  else if (currentMedia && currentTrack.status == 3)
                    {
                      resume();
                    }


                  //if (currentTrack.id == trackID)
                  //  {
                  //    if (myStatus == 2)
                  //      {
                  //        pause();
                  //      } else
                  //      {
                  //        //if (currentTrack.status == Media.MEDIA_PAUSED) {
                  //        resume();
                  //        //}
                  //      }
                  //    return;
                  //  } else
                  //  {
                  //    if (currentTrack.id > -1)
                  //      {
                  //        stop();
                  //      }
                  //  }
                }

              $timeout(function ()
              {
                checkMedia();
                play(tracks[trackID], trackID);
              }, 300);

            },

          pause: function ()
            {
              console.log('pause 1=========');
              pause();
              //kill();
            },
          stop: function ()
            {
              destroy();
            },
          seekTo: function (pos)
            {
              seekTo(pos);
            },

          destroy: function ()
            {
              destroy();
            }

        }

      }
  ])
  .directive('ionAudioTrack', [
    'MediaManager', '$rootScope', function (MediaManager, $rootScope)
      {
        return {
          transclude: true,
          template: '<ng-transclude></ng-transclude>',
          restrict: 'E',
          scope: {
            track: '='
          },
          controller: [
            '$scope', function ($scope)
              {
                var controller = this;

                $scope.track.progress = 0;
                $scope.track.status = 0;
                $scope.track.duration = -1;

                var playbackSuccess = function ()
                  {
                    $scope.track.status = 0;
                    $scope.track.progress = 0;
                  };
                var statusChange = function (status)
                  {
                    $scope.track.status = status;
                  };
                var progressChange = function (progress, duration)
                  {
                    $scope.track.progress = progress;
                    $scope.track.duration = duration;
                  };
                var notifyProgressBar = function ()
                  {
                    $rootScope.$broadcast('ionic-audio:trackChange', $scope.track);
                  };

                this.seekTo = function (pos)
                  {
                    MediaManager.seekTo(pos);
                  };

                this.play = function ()
                  {
                    if (!MediaManager) return;

                    MediaManager.play($scope.track.id);

                    // notify global progress bar if detached from track
                    if (!controller.hasOwnProgressBar) notifyProgressBar();

                    return $scope.track.id;
                  };
                this.stop = function ()
                  {
                    MediaManager.destroy();
                  };
                this.getTrack = function ()
                  {
                    return $scope.track;
                  };

                if (MediaManager)
                  {
                    $scope.track.id = MediaManager.add($scope.track, playbackSuccess, null, statusChange,
                      progressChange);
                  }
              }
          ],
          link: function (scope, element, attrs, controller)
            {
              controller.hasOwnProgressBar = element.find('ion-audio-progress-bar').length > 0;

              scope.$on('$destroy', function ()
              {
                MediaManager.destroy();
              });
            }
        }
      }
  ])
  .directive('ionAudioControls', [
    function ()
      {
        return {
          restrict: 'EA',
          scope: {},
          require: ['ionAudioControls', '^^ionAudioTrack'],
          controller: [
            '$scope', '$element', function ($scope, $element)
              {
                var spinnerElem = $element.find('ion-spinner'), hasLoaded, self = this;
                spinnerElem.addClass('ng-hide');

                this.toggleSpinner = function ()
                  {
                    spinnerElem.toggleClass('ng-hide');
                  };

                this.playTrack = function ()
                  {
                    if (!hasLoaded)
                      {
                        //self.toggleSpinner();
                      }
                    self.play();
                  };
                this.stopTrack = function ()
                  {
                    self.destroy();
                  };

                var unbindStatusListener = $scope.$watch('track.status', function (status)
                {
                  switch (status)
                  {
                    case 1: // Media.MEDIA_STARTING
                      hasLoaded = false;
                      break;
                    case 2: // Media.MEDIA_RUNNING
                      if (!hasLoaded)
                        {
                          //self.toggleSpinner();
                          hasLoaded = true;
                        }
                      break;
                    case 3: // Media.MEDIA_PAUSED
                      break;
                    case 0: // Media.MEDIA_NONE
                    case 4: // Media.MEDIA_STOPPED
                      hasLoaded = false;
                      break;
                  }
                });

                $scope.$on('$destroy', function ()
                {
                  unbindStatusListener();
                });
              }
          ],
          link: function (scope, element, attrs, controllers)
            {
              var ionAudioTrackCtrl = controllers[1];
              controllers[0].play = ionAudioTrackCtrl.play;

              scope.track = ionAudioTrackCtrl.getTrack();
            }
        }
      }
  ])
  .directive('ionAudioPlay', [
    function ()
      {
        return {
          //scope: true,
          restrict: 'A',
          require: ['^^ionAudioTrack', '^^ionAudioControls'],
          link: function (scope, element, attrs, controllers)
            {
              var isLoading, isPlaying, currentStatus = 0;

              scope.track = controllers[0].getTrack();

              var controller = controllers[1];
              console.log('controller==============', controller);
              var init = function ()
                {
                  isLoading = false;
                  isPlaying = false;
                  element.addClass('ion-play');
                  element.removeClass('ion-pause');
                  element.text(attrs.textPlay);
                };

              var setText = function ()
                {
                  if (!attrs.textPlay || !attrs.textPause) return;

                  element.text((element.text() == attrs.textPlay ? attrs.textPause : attrs.textPlay));
                };

              var togglePlaying = function ()
                {
                  element.toggleClass('ion-play ion-pause');
                  setText();
                };

              element.on('click', function ()
              {
                console.log('element on click');
                //if (isLoading) return;  //  debounce multiple clicks
                if (isPlaying)
                  {
                    console.log('element isplaying == true', isPlaying);
                    isPlaying = false;
                    return controller.stopTrack();
                  }
                controller.playTrack();
                isPlaying = true;
                togglePlaying();
                if (currentStatus == 0) isLoading = true;
              });

              var unbindStatusListener = scope.$watch('track.status', function (status)
              {
                //  Media.MEDIA_NONE or Media.MEDIA_STOPPED
                if (status == 0 || status == 4)
                  {
                    init();
                  } else if (status == 2)
                  {   // Media.MEDIA_RUNNING
                    isLoading = false;
                  }

                currentStatus = status;
              });

              init();

              scope.$on('$destroy', function ()
              {
                unbindStatusListener();
              });
            }
        }
      }
  ])
  .directive('ionAudioProgressBar', [
    'MediaManager', function (MediaManager)
      {
        return {
          restrict: 'E',
          template: '<h2 class="ion-audio-track-info" ng-style="displayTrackInfo()">{{track.title}} - {{track.artist}}</h2>'
          +
          '<div class="range">' +
          '<ion-audio-progress track="track"></ion-audio-progress>' +
          '<input type="range" name="volume" min="0" max="{{track.duration}}" ng-model="track.progress" on-release="sliderRelease()" disabled>'
          +
          '<ion-audio-duration track="track"></ion-audio-duration>' +
          '</div>',
          require: '?^^ionAudioTrack',
          scope: {},
          link: function (scope, element, attrs, controller)
            {
              var slider = element.find('input'), unbindTrackListener;

              scope.track = {
                progress: 0,
                status: 0,
                duration: -1
              };

              if (!angular.isDefined(attrs.displayTime))
                {
                  element.find('ion-audio-progress').remove();
                  element.find('ion-audio-duration').remove();
                }
              if (!angular.isDefined(attrs.displayInfo))
                {
                  element.find('h2').remove();
                }

              // hide/show track info if available
              scope.displayTrackInfo = function ()
                {
                  return {
                    visibility: angular.isDefined(attrs.displayInfo) && (scope.track.title || scope.track.artist) ?
                      'visible' : 'hidden'
                  }
                };

              // disable slider if track is not playing
              var unbindStatusListener = scope.$watch('track.status', function (status)
              {
                // disable if track hasn't loaded
                slider.prop('disabled', status == 0);   //   Media.MEDIA_NONE
              });

              if (controller)
                {
                  // get track from parent audio track directive
                  scope.track = controller.getTrack();
                } else
                {
                  // get track from current playing track elsewhere in the DOM
                  unbindTrackListener = scope.$on('ionic-audio:trackChange', function (e, track)
                  {
                    console.log('trackChange', track);
                    scope.track = track;
                  });
                }

              // handle track seek-to
              scope.sliderRelease = function ()
                {
                  var pos = scope.track.progress;
                  MediaManager.seekTo(pos);
                };

              scope.$on('$destroy', function ()
              {
                unbindStatusListener();
                if (angular.isDefined(unbindTrackListener))
                  {
                    unbindTrackListener();
                  }
              });
            }
        }
      }
  ])
  .directive('ionAudioProgress', [
    function ()
      {
        return {
          restrict: 'E',
          scope: {
            track: '='
          },
          template: '{{track.progress | time}}'
        }
      }
  ])
  .directive('ionAudioDuration', [
    function ()
      {
        return {
          restrict: 'E',
          scope: {
            track: '='
          },
          template: '{{track.duration | duration}}'
        }
      }
  ]);
